/* This testcase is part of GDB, the GNU debugger.

   Copyright 2010, 2011 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

/* The original program corresponding to watch-notconst2.S.

   This program is not compiled; the .S version is used instead.

   The purpose of this test is to see if GDB can still watch the
   variable `x' even when we compile the program using -O2
   optimization.  */

extern int g (int j);

int
f (int i)
{
  int x = 5;
  g (2);
  x = i;
  return g (x);
}
